@@ -15,8 +15,6 @@ from .normalizationNode import NormalizationNode
 from .addSignIndicator import AddSignIndicatorNode
 from .linearEmbedding import LinearEmbeddingNode
 from .onehotEncoding import OneHotEncodingNode
-from .daskComputeNode import DaskComputeNode
-from .persistNode import PersistNode
 
 __all__ = ["AverageNode", "AssetFilterNode", "LeftMergeNode",
            "ReturnFeatureNode", "SortNode",
@@ -24,4 +22,4 @@ __all__ = ["AverageNode", "AssetFilterNode", "LeftMergeNode",
            "ValueFilterNode", "RenameNode", "AssetIndicatorNode",
            "DropNode", "IndicatorNode", "NormalizationNode",
            "AddSignIndicatorNode", "LinearEmbeddingNode",
-           "OneHotEncodingNode", "DaskComputeNode", "PersistNode"]
+           "OneHotEncodingNode"]
